type Fn = {
    name: string;
    fn: (...args: any) => any;
};
type CallContext = {
    that: unknown;
    args: IArguments;
};
type ExecuteState = {
    state: 'execute';
};
type ErrorState = {
    state: 'error';
    error: unknown;
};
type NormalState = {
    state: 'result';
    result: unknown;
};
type InstrumentHandler = {
    name?: string;
    onExecute?: (context: Fn & CallContext & ExecuteState) => void | Partial<CallContext & Fn & ExecuteState>;
    onError?: (context: Fn & CallContext & ErrorState & {
        isPromise: boolean;
    }) => void | ErrorState | NormalState;
    onResult?: (context: Fn & CallContext & NormalState & {
        isPromise: boolean;
    }) => void | ErrorState | NormalState;
};
type CreateInstrumentHandler = InstrumentHandler | (() => InstrumentHandler);
declare const createInstrument: (opts: CreateInstrumentHandler) => InstrumentHandler;
type InstrumentFunction = <X extends Fn['fn']>(fn: X, name?: string) => X;
declare function composeInstrument(...ins: CreateInstrumentHandler[]): InstrumentFunction | undefined;

type Provider<Provide, Input = unknown> = (() => Provide | Promise<Provide>) | ((input: Input) => Provide | Promise<Provide>);
type ProviderOption = {
    name?: string;
    instrument?: InstrumentFunction;
    mode?: 'prototype' | 'singleton';
};
declare const defaultProviderOptions: ProviderOption;
declare function setInstrument(inst: CreateInstrumentHandler): void;
type Executor<Provide, Dependent = unknown> = {
    execute<Output>(executable: Provider<Output, Provide>, options?: ProviderOption): Promise<Output>;
    get(): Promise<Provide>;
    prepare<Input extends Array<any>, Output>(provider: (provide: Provide, ...input: Input) => Output, options?: ProviderOption): (...input: Input) => Promise<Awaited<ReturnType<typeof provider>>>;
    _inject: (executor: Executor<Dependent>) => void;
};
declare function create<Provide>(provider: Provider<Provide>, options?: ProviderOption): Executor<Provide>;
declare function create<Provide, Dependent>(provider: Provider<Provide, Dependent>, dependent?: Executor<Dependent>, options?: ProviderOption): Executor<Provide>;
declare const value: <Provide>(value: Provide, options?: Omit<ProviderOption, 'mode'>) => Executor<Provide, unknown>;
declare const prepare: <Dependent, Input extends any[], Output>(provider: (provide: Dependent, ...input: Input) => Output, dep: Executor<Dependent, unknown>) => (...input: Input) => Promise<Awaited<Output>>;
declare function execute<Output>(executable: Provider<Output>, options?: ProviderOption): Promise<Output>;
declare function execute<Output, Dependent>(executable: Provider<Output, Dependent>, dependent: Executor<Dependent>, options?: ProviderOption): Promise<Output>;
type inferProvide<T> = T extends Executor<infer S> ? S : never;
declare const combine: <L extends Record<string, Executor<any, unknown>>>(layout: L) => Executor<{ [key in keyof L]: inferProvide<L[key]>; }, unknown>;

export { CreateInstrumentHandler, Executor, InstrumentHandler, ProviderOption, combine, composeInstrument, create, createInstrument, defaultProviderOptions, execute, inferProvide, prepare, setInstrument, value };
