var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  combine: () => combine,
  composeInstrument: () => composeInstrument,
  create: () => create,
  createInstrument: () => createInstrument,
  defaultProviderOptions: () => defaultProviderOptions,
  execute: () => execute,
  prepare: () => prepare,
  setInstrument: () => setInstrument,
  value: () => value
});
module.exports = __toCommonJS(src_exports);

// src/instrument.ts
function isPromise(value2) {
  return value2 === Promise.resolve(value2);
}
var createInstrument = (opts) => {
  return typeof opts === "function" ? opts() : opts;
};
function createInstrumentor(opts) {
  return function(fn, fnName) {
    let name = opts?.name || fn.name || fnName || "anonymous";
    const handler = typeof opts === "function" ? opts() : opts;
    const f = function() {
      let that = this;
      let args = arguments;
      let func = fn;
      try {
        const executeState = handler.onExecute?.({ name, fn, that, args, state: "execute" });
        if (executeState !== void 0) {
          that = executeState.that || that;
          args = executeState.args || args;
          func = executeState.fn || func;
          name = executeState.name || name;
        }
        let result = func.apply(that, args);
        if (isPromise(result)) {
          return result.then((next) => {
            const promiseResultState = handler.onResult?.({ name, fn: func, that, args, state: "result", result: next, isPromise: true });
            if (promiseResultState?.state === "result") {
              return promiseResultState.result;
            }
            if (promiseResultState?.state === "error") {
              throw promiseResultState.error;
            }
            return next;
          }).catch((error) => {
            const errorState = handler.onError?.({ name, fn: func, that, args, error, state: "error", isPromise: true });
            if (errorState?.state === "error") {
              throw errorState.error;
            }
            if (errorState?.state === "result") {
              return errorState.result;
            }
            throw error;
          });
        } else {
          const resultState = handler.onResult?.({ name, fn: func, that, args, state: "result", result: void 0, isPromise: false });
          if (resultState?.state === "error") {
            throw resultState.error;
          }
          if (resultState?.state === "result") {
            return resultState.result;
          }
          return result;
        }
      } catch (error) {
        const errorState = handler.onError?.({ name, fn: func, that, args, error, state: "error", isPromise: false });
        if (errorState?.state === "error") {
          throw errorState.error;
        }
        if (errorState?.state === "result") {
          return errorState.result;
        }
        throw error;
      }
    };
    Object.defineProperty(f, "name", {
      value: name,
      writable: false
    });
    return f;
  };
}
function instrument(container, instrumentFn, maxLevel = 1, currentLevel = 0) {
  if (container === null || container === void 0)
    return container;
  if (currentLevel > maxLevel)
    return container;
  if (
    // there can be more
    typeof container !== "object" || Array.isArray(container)
  ) {
    return container;
  }
  Object.keys(container).forEach((key) => {
    const instance = container[key];
    if (typeof instance === "function") {
      container[key] = instrumentFn(instance, key);
    } else {
      container[key] = instrument(container[key], instrumentFn, maxLevel, currentLevel + 1);
    }
  });
  return container;
}
function nextInstrument(prev, inst) {
  const next = createInstrumentor(createInstrument(inst));
  const mixin = (fn, name) => {
    const instrumented = prev?.(fn, name) || fn;
    return next(instrumented, name);
  };
  return mixin;
}
function composeInstrument(...ins) {
  return ins.reduceRight((prev, next) => {
    return nextInstrument(prev, next);
  }, void 0);
}

// src/index.ts
var defaultProviderOptions = {
  mode: "singleton",
  instrument: createInstrumentor({})
};
function setInstrument(inst) {
  defaultProviderOptions.instrument = nextInstrument(defaultProviderOptions.instrument, inst);
}
function isExecutor(obj) {
  return typeof obj?.["get"] === "function";
}
function extract(secondParam, thirdParam) {
  return isExecutor(secondParam) ? { dependent: secondParam, options: thirdParam } : { dependent: void 0, options: secondParam || thirdParam };
}
function set(target, name, value2) {
  Object.defineProperty(target, name, { value: value2, writable: false });
}
function create(provider, secondParam, thirdParam) {
  const { dependent, options } = extract(secondParam, thirdParam);
  const opts = { ...defaultProviderOptions, ...options };
  const name = opts?.name || provider.name || "anonymous";
  let cached = void 0;
  let dependentRef = dependent;
  async function load() {
    const actualized = dependentRef ? await dependentRef.get() : void 0;
    const provide = await provider(actualized);
    return { provide };
  }
  async function init() {
    if (opts.mode === "prototype") {
      return load();
    } else {
      if (cached === void 0) {
        cached = load();
      }
    }
    return cached;
  }
  const executor = {
    execute: async (provider2, options2) => create(async () => provider2(await executor.get()), options2 || { name: `${name}.execute` }).get(),
    get: async () => (await init()).provide,
    prepare(provider2, options2) {
      return async function() {
        const that = this;
        const args = arguments;
        return execute(async () => provider2.call(that, await executor.get(), ...args), options2);
      };
    },
    _inject: async (executor2) => {
      dependentRef = executor2;
    }
  };
  set(executor.get, "name", `${name}.get`);
  set(executor.execute, "name", `${name}.execute`);
  set(executor.prepare, "name", `${name}.prepare`);
  if (opts.instrument) {
    instrument(executor, opts.instrument);
  }
  return executor;
}
var value = (value2, options) => create(() => value2, options);
var prepare = (provider, dep) => dep.prepare(provider);
async function execute(executable, secondParam, thirdParam) {
  const { dependent, options } = extract(secondParam, thirdParam);
  return dependent ? dependent.execute(executable, options) : create(executable, options).get();
}
var combine = function(layout) {
  return create(async () => {
    const layoutPromise = Object.entries(layout).map(([key, executor]) => executor.get().then((value2) => [key, value2]));
    const result = Object.fromEntries(await Promise.all(layoutPromise));
    return result;
  });
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  combine,
  composeInstrument,
  create,
  createInstrument,
  defaultProviderOptions,
  execute,
  prepare,
  setInstrument,
  value
});
//# sourceMappingURL=index.js.map